"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var _a, _b;
var types_1 = require("./types/types");
var FilterOperator = types_1.Query.FilterOperator;
var LogicFilterOperator = types_1.Query.LogicFilterOperator;
var filterOperatorMap = (_a = {},
    _a[FilterOperator.eq] = function (fieldName) { return "= :" + fieldName; },
    _a[FilterOperator.ne] = function (fieldName) { return "!= :" + fieldName; },
    _a[FilterOperator.gt] = function (fieldName) { return "> :" + fieldName; },
    _a[FilterOperator.gte] = function (fieldName) { return ">= :" + fieldName; },
    _a[FilterOperator.lt] = function (fieldName) { return "< :" + fieldName; },
    _a[FilterOperator.lte] = function (fieldName) { return "<= :" + fieldName; },
    _a[FilterOperator.in] = function (fieldName) { return "IN (:" + fieldName + ")"; },
    _a[FilterOperator.nin] = function (fieldName) { return "NOT IN (:" + fieldName + "})"; },
    _a[FilterOperator.regex] = function (fieldName) { return "~ :" + fieldName; },
    _a[FilterOperator.like] = function (fieldName) { return "~* :" + fieldName; },
    _a);
var logicOperatorMap = (_b = {},
    _b[LogicFilterOperator.or] = function (qb) { return qb.orWhere.bind(qb); },
    _b[LogicFilterOperator.and] = function (qb) { return qb.andWhere.bind(qb); },
    _b);
var logicOperatorMapKeys = Object.keys(logicOperatorMap);
var isReservedKeyWord = function (key) {
    return Object.keys(filterOperatorMap).includes(key) || Object.keys(logicOperatorMap).includes(key);
};
function applyToQueryBuilder(filters, filterFieldName, parent, filterValue, queryBuilder, logicOperatorFn) {
    var _a;
    if (logicOperatorMapKeys.includes(filterFieldName)) {
        var logicOperatorFn_1 = logicOperatorMap[filterFieldName](queryBuilder);
        applyFilters(queryBuilder, filters[filterFieldName], logicOperatorFn_1);
        return;
    }
    else if (Object.keys(filterOperatorMap).includes(filterFieldName)) {
        var parentOrFieldName = parent ? parent.name : filterFieldName;
        var operator = filterOperatorMap[filterFieldName](parentOrFieldName);
        logicOperatorFn(parentOrFieldName + " " + operator, (_a = {},
            _a[parentOrFieldName] = filterValue,
            _a));
        return;
    }
}
exports.applyToQueryBuilder = applyToQueryBuilder;
function applyFilters(queryBuilder, filters, logicOperatorFn, parent) {
    if (logicOperatorFn === void 0) { logicOperatorFn = queryBuilder.where.bind(queryBuilder); }
    if (!filters || typeof filters !== 'object') {
        return;
    }
    if (Array.isArray(filters)) {
        filters.forEach(function (filter) { return applyFilters(queryBuilder, filter, logicOperatorFn, parent); });
        return;
    }
    Object.keys(filters).forEach(function (filterFieldName, index) {
        if (index === 1) {
            logicOperatorFn = queryBuilder.andWhere.bind(queryBuilder);
        }
        var filter = filters[filterFieldName];
        if (isReservedKeyWord(filterFieldName)) {
            var filterValue = filters[filterFieldName];
            applyToQueryBuilder(filters, filterFieldName, parent, filterValue, queryBuilder, logicOperatorFn);
            return;
        }
        if (Array.isArray(filter)) {
            filter.forEach(function (f) {
                return applyFilters(queryBuilder, f, logicOperatorFn, {
                    filters: filters,
                    name: filterFieldName,
                });
            });
            return;
        }
        if (filter && typeof filter === 'object') {
            applyFilters(queryBuilder, filter, logicOperatorFn, {
                filters: filters,
                name: filterFieldName,
            });
        }
    });
}
exports.applyFilters = applyFilters;
function filter(rootValue, args, context) {
    applyFilters(context.queryBuilder, args.filter);
    return context.queryBuilder;
}
exports.filter = filter;
//# sourceMappingURL=filter.js.map