"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const path = require("path");
const fs = require("fs");
const shell = require("shelljs");
const chalk_1 = require("chalk");
const Generator = require("yeoman-generator");
const ora = require("ora");
const graphQLLogo_1 = require("./graphQLLogo");
const tic = chalk_1.default.green('✓');
const tac = chalk_1.default.red('✗');
class AppGenerator extends Generator {
    constructor(args, options) {
        super(args, options);
        this.argument('name', {
            type: String,
            required: true,
        });
        this.dir = path.resolve(this.options.name);
    }
    initializing() {
        this.spinner = ora();
        this._printGraphQLLogo();
    }
    cloneStarterCode() {
        this.spinner.start();
        this._validateDirectory();
        this.spinner.text = 'Creating a new GraphQL project...';
        const repository = 'https://gitlab.com/create-graphql-node/boilerplate-ts.git';
        const done = this.async();
        const command = 'git';
        const commandOpts = ['clone', repository, this.dir];
        const checkoutCommandOpts = ['checkout', 'master'];
        this.spawnCommand(command, commandOpts, { stdio: 'ignore' }).on('close', () => {
            shell.cd(this.dir);
            this.spawnCommand(command, checkoutCommandOpts, { stdio: 'ignore' }).on('close', () => {
                this.spinner.stop();
                this.log(`${tic} GraphQL project ${this.options.name} created.`);
                done();
            });
        });
    }
    _printGraphQLLogo() {
        this.log(graphQLLogo_1.graphQlLogo);
    }
    _validateDirectory() {
        try {
            fs.lstatSync(this.dir).isDirectory();
            this._logAndExit(`${tac} Directory "${this.options.name}" already exists,
        please enter a new directory name or delete "${this.options.name}"!`);
            return false;
        }
        catch (err) {
            return true;
        }
    }
    installModules() {
        shell.cd(this.dir);
        this.spinner.start();
        this.spinner.text = 'Installing dependencies...';
        const done = this.async();
        const command = './cli';
        const args = ['setup'];
        this.spawnCommand(command, args, { stdio: 'inherit' }).on('close', () => {
            this.spinner.stop();
            done();
        });
    }
    _cleanDir() {
        shell.cd(this.dir);
        shell.rm('-rf', '.git');
    }
    _logAndExit(message) {
        this.spinner.stop();
        this.log(message);
        process.exit(1);
    }
    end() {
        this._cleanDir();
        this.log(`${tic} Your new project with GraphQL has been created! 🔥`);
    }
}
module.exports = AppGenerator;
//# sourceMappingURL=app.js.map