"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
const __1 = require("..");
var FilterOperator = __1.Query.FilterOperator;
var LogicFilterOperator = __1.Query.LogicFilterOperator;
const filterOperatorMap = {
    [FilterOperator.eq]: fieldName => `= :${fieldName}`,
    [FilterOperator.ne]: fieldName => `!= :${fieldName}`,
    [FilterOperator.gt]: fieldName => `> :${fieldName}`,
    [FilterOperator.gte]: fieldName => `>= :${fieldName}`,
    [FilterOperator.lt]: fieldName => `< :${fieldName}`,
    [FilterOperator.lte]: fieldName => `<= :${fieldName}`,
    [FilterOperator.in]: fieldName => `IN (:${fieldName})`,
    [FilterOperator.nin]: fieldName => `NOT IN (:${fieldName}})`,
    [FilterOperator.regex]: fieldName => `~ :${fieldName}`,
    [FilterOperator.like]: fieldName => `~* :${fieldName}`,
};
const logicOperatorMap = {
    [LogicFilterOperator.or]: qb => qb.orWhere.bind(qb),
    [LogicFilterOperator.and]: qb => qb.andWhere.bind(qb),
};
const logicOperatorMapKeys = Object.keys(logicOperatorMap);
const isReservedKeyWord = (key) => Object.keys(filterOperatorMap).includes(key) || Object.keys(logicOperatorMap).includes(key);
function applyToQueryBuilder(filters, filterFieldName, parent, filterValue, queryBuilder, logicOperatorFn) {
    if (logicOperatorMapKeys.includes(filterFieldName)) {
        const logicOperatorFn = logicOperatorMap[filterFieldName](queryBuilder);
        applyFilters(queryBuilder, filters[filterFieldName], logicOperatorFn);
        return;
    }
    else if (Object.keys(filterOperatorMap).includes(filterFieldName)) {
        const parentOrFieldName = parent ? parent.name : filterFieldName;
        const operator = filterOperatorMap[filterFieldName](parentOrFieldName);
        logicOperatorFn(`${parentOrFieldName} ${operator}`, {
            [parentOrFieldName]: filterValue,
        });
        return;
    }
}
exports.applyToQueryBuilder = applyToQueryBuilder;
function applyFilters(queryBuilder, filters, logicOperatorFn = queryBuilder.where.bind(queryBuilder), parent) {
    if (!filters || typeof filters !== 'object') {
        return;
    }
    if (Array.isArray(filters)) {
        filters.forEach(filter => applyFilters(queryBuilder, filter, logicOperatorFn, parent));
        return;
    }
    Object.keys(filters).forEach((filterFieldName, index) => {
        if (index === 1) {
            logicOperatorFn = queryBuilder.andWhere.bind(queryBuilder);
        }
        const filter = filters[filterFieldName];
        if (isReservedKeyWord(filterFieldName)) {
            const filterValue = filters[filterFieldName];
            applyToQueryBuilder(filters, filterFieldName, parent, filterValue, queryBuilder, logicOperatorFn);
            return;
        }
        if (Array.isArray(filter)) {
            filter.forEach(f => applyFilters(queryBuilder, f, logicOperatorFn, {
                filters,
                name: filterFieldName,
            }));
            return;
        }
        if (filter && typeof filter === 'object') {
            applyFilters(queryBuilder, filter, logicOperatorFn, {
                filters: filters,
                name: filterFieldName,
            });
        }
    });
}
exports.applyFilters = applyFilters;
function filter(rootValue, args, context) {
    applyFilters(context.queryBuilder, args.filter);
    return context.queryBuilder;
}
exports.filter = filter;
//# sourceMappingURL=filter.js.map