"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
const errors_1 = require("./errors");
const lodash_1 = require("lodash");
function compose(...fns) {
    return fns.reduceRight((prevFn, nextFn) => (value) => nextFn(prevFn(value)), (value) => value);
}
exports.compose = compose;
function withName(name) {
    return function (target) {
        return Object.entries(target).reduce((acc, [key, resolverFn]) => (Object.assign({}, acc, { [`${key}${name}`]: resolverFn })), {});
    };
}
exports.withName = withName;
function just(scopes) {
    return (target) => Object.entries(target).reduce((acc, [key, resolverFn]) => {
        if ((Array.isArray(scopes) && !scopes.includes(key)) ||
            (!Array.isArray(scopes) && scopes !== key)) {
            return acc;
        }
        return Object.assign({}, acc, { [key]: resolverFn });
    }, {});
}
exports.just = just;
function defaultMutator(model, entity, { [model.name.toLowerCase()]: input }, options) {
    return __awaiter(this, void 0, void 0, function* () {
        yield Promise.all(Object.entries(input).map(([key, value]) => __awaiter(this, void 0, void 0, function* () {
            if (options.keyBlackList.includes(key)) {
                return;
            }
            if (typeof options.mapper !== 'function') {
                entity[key] = value;
                return;
            }
            const mapped = yield options.mapper(value, key, entity);
            if (mapped !== undefined) {
                entity[key] = mapped;
            }
        })));
    });
}
function defaultMapper(value, field, entity) {
    return __awaiter(this, void 0, void 0, function* () {
        if (typeof entity[field] === 'function') {
            yield entity[field](value);
            return;
        }
        return value;
    });
}
const defaultMutators = {
    create: { fn: defaultMutator, options: { keyBlackList: [] } },
    update: { fn: defaultMutator, options: { keyBlackList: ['id'] } },
};
const defaultMappers = {
    create: defaultMapper,
    update: defaultMapper,
};
function generateMutations(model, options) {
    const { hooks, mutators, mappers } = lodash_1.merge({
        hooks: {
            preCreate: null,
            postCreate: null,
            preUpdate: null,
            postUpdate: null,
            preRemove: null,
            postRemove: null,
        },
        mutators: defaultMutators,
        mappers: defaultMappers,
    }, options || {});
    return {
        create: (rootValue, args, context, info) => __awaiter(this, void 0, void 0, function* () {
            const repository = context.connection.getRepository(model);
            const entity = repository.create();
            if (typeof hooks.preCreate === 'function') {
                yield hooks.preCreate(entity, rootValue, args, context, info);
            }
            yield mutators.create.fn(model, entity, args, Object.assign({ mapper: mappers.create }, mutators.create.options));
            const savedEntity = yield repository.save(entity);
            if (typeof hooks.postCreate === 'function') {
                yield hooks.postCreate(savedEntity, rootValue, args, context, info);
            }
            return savedEntity;
        }),
        update: (rootValue, args, context, info) => __awaiter(this, void 0, void 0, function* () {
            const { id } = args;
            const repository = context.connection.getRepository(model);
            const entity = yield repository.findOne(id);
            if (!entity) {
                throw new errors_1.NotFoundError(`Could not find ${model.name} for id ${id}`);
            }
            if (typeof hooks.preUpdate === 'function') {
                yield hooks.preUpdate(entity, rootValue, args, context, info);
            }
            yield mutators.update.fn(model, entity, args, Object.assign({ mapper: mappers.update }, mutators.update.options));
            const savedEntity = yield repository.save(entity);
            if (typeof hooks.postUpdate === 'function') {
                yield hooks.postUpdate(savedEntity, rootValue, args, context, info);
            }
            return savedEntity;
        }),
        remove: (rootValue, args, context, info) => __awaiter(this, void 0, void 0, function* () {
            const { [model.name.toLowerCase()]: { id }, } = args;
            const repository = context.connection.getRepository(model);
            const entity = yield repository.findOne(id);
            if (!entity) {
                throw new errors_1.NotFoundError(`Could not find ${model.name} for id ${id}`);
            }
            if (typeof hooks.preRemove === 'function') {
                yield hooks.preRemove(entity, rootValue, { id }, context, info);
            }
            const removedEntity = yield repository.remove(entity);
            if (typeof hooks.postRemove === 'function') {
                yield hooks.postRemove(removedEntity, rootValue, { id }, context, info);
            }
            return Object.assign({}, entity, { id });
        }),
    };
}
exports.generateMutations = generateMutations;
//# sourceMappingURL=generateMutations.js.map