import { BaseEntity, Repository, SelectQueryBuilder } from 'typeorm';
import { IContext, Query } from '..';
import IPaginationArguments = Query.IPaginationArguments;
export declare const base64: (str: string) => string;
export declare const unbase64: (b64: string) => string;
/**
 * Rederives the offset from the cursor string
 */
export declare function cursorToOffset(cursor: string): number;
/**
 * Given an optional cursor and a default offset, returns the offset to use;
 * if the cursor contains a valid offset, that will be used, otherwise it will
 * be the default.
 */
export declare function getOffsetWithDefault(cursor: string, defaultOffset: number): number;
/**
 * Creates the cursor string from an offset.
 */
export declare function offsetToCursor(offset: number): string;
interface IPaginateContext extends IContext {
    repository: Repository<BaseEntity>;
    queryBuilder?: SelectQueryBuilder<BaseEntity>;
    mapper?: (value: BaseEntity, index: number, array: BaseEntity[]) => any;
}
/**
 * Accepts a mongoose query and connection arguments, and returns a connection
 * object for use in GraphQL. It uses array offsets as pagination, so pagination
 * will work only if the data set is static.
 */
export declare function paginate(rootValue: any, args: IPaginationArguments, context: IPaginateContext): Promise<{
    edges: {
        cursor: string;
        node: BaseEntity;
    }[];
    pageInfo: {
        startCursor: string;
        endCursor: string;
        hasPreviousPage: boolean;
        hasNextPage: boolean;
        count: number;
    };
}>;
export {};
