"use strict";
var __assign = (this && this.__assign) || Object.assign || function(t) {
    for (var s, i = 1, n = arguments.length; i < n; i++) {
        s = arguments[i];
        for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p))
            t[p] = s[p];
    }
    return t;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : new P(function (resolve) { resolve(result.value); }).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __generator = (this && this.__generator) || function (thisArg, body) {
    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
    return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function() { return this; }), g;
    function verb(n) { return function (v) { return step([n, v]); }; }
    function step(op) {
        if (f) throw new TypeError("Generator is already executing.");
        while (_) try {
            if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;
            if (y = 0, t) op = [op[0] & 2, t.value];
            switch (op[0]) {
                case 0: case 1: t = op; break;
                case 4: _.label++; return { value: op[1], done: false };
                case 5: _.label++; y = op[1]; op = [0]; continue;
                case 7: op = _.ops.pop(); _.trys.pop(); continue;
                default:
                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }
                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }
                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }
                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }
                    if (t[2]) _.ops.pop();
                    _.trys.pop(); continue;
            }
            op = body.call(thisArg, _);
        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }
        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };
    }
};
Object.defineProperty(exports, "__esModule", { value: true });
var PREFIX = 'connection:';
exports.base64 = function (str) { return new Buffer(str, 'ascii').toString('base64'); };
exports.unbase64 = function (b64) { return new Buffer(b64, 'base64').toString('ascii'); };
/**
 * Rederives the offset from the cursor string
 */
function cursorToOffset(cursor) {
    return parseInt(exports.unbase64(cursor).substring(PREFIX.length), 10);
}
exports.cursorToOffset = cursorToOffset;
/**
 * Given an optional cursor and a default offset, returns the offset to use;
 * if the cursor contains a valid offset, that will be used, otherwise it will
 * be the default.
 */
function getOffsetWithDefault(cursor, defaultOffset) {
    if (cursor === undefined) {
        return defaultOffset;
    }
    var offset = cursorToOffset(cursor);
    return isNaN(offset) ? defaultOffset : offset;
}
exports.getOffsetWithDefault = getOffsetWithDefault;
/**
 * Creates the cursor string from an offset.
 */
function offsetToCursor(offset) {
    return exports.base64(PREFIX + offset);
}
exports.offsetToCursor = offsetToCursor;
/**
 * Accepts a mongoose query and connection arguments, and returns a connection
 * object for use in GraphQL. It uses array offsets as pagination, so pagination
 * will work only if the data set is static.
 */
function paginate(rootValue, args, context) {
    return __awaiter(this, void 0, void 0, function () {
        var _a, mapper, repository, _b, queryBuilder, after, before, first, last, sort, totalCount, beforeOffset, afterOffset, startOffset, endOffset, skip, limit, typeOrmSort, slice, count, edges, firstEdge, lastEdge, lowerBound, upperBound;
        return __generator(this, function (_c) {
            switch (_c.label) {
                case 0:
                    mapper = context.mapper, repository = context.repository, _b = context.queryBuilder, queryBuilder = _b === void 0 ? repository.createQueryBuilder() : _b;
                    after = args.after, before = args.before, first = args.first, last = args.last, sort = args.sort;
                    return [4 /*yield*/, repository.count()];
                case 1:
                    totalCount = _c.sent();
                    beforeOffset = getOffsetWithDefault(before, totalCount);
                    afterOffset = getOffsetWithDefault(after, -1);
                    startOffset = Math.max(-1, afterOffset) + 1;
                    endOffset = Math.min(totalCount, beforeOffset);
                    if (first) {
                        endOffset = Math.min(endOffset, startOffset + first);
                    }
                    if (last) {
                        startOffset = Math.max(startOffset, endOffset - last);
                    }
                    skip = Math.max(startOffset, 0);
                    limit = endOffset - startOffset;
                    if (Array.isArray(sort)) {
                        typeOrmSort = sort.reduce(function (sortObject, s) {
                            var _a;
                            return (__assign({}, sortObject, (_a = {}, _a[s.field] = s.order.toLowerCase(), _a)));
                        }, {});
                        queryBuilder.orderBy(typeOrmSort);
                    }
                    // If supplied slice is too large, trim it down before mapping over it.
                    queryBuilder.skip(skip).take(limit);
                    slice = [];
                    count = 0;
                    if (!(limit > 0)) return [3 /*break*/, 3];
                    return [4 /*yield*/, queryBuilder.getManyAndCount()];
                case 2:
                    _a = _c.sent(), slice = _a[0], count = _a[1];
                    _c.label = 3;
                case 3:
                    // If we have a mapper function, map it!
                    if (typeof mapper === 'function') {
                        slice = slice.map(mapper);
                    }
                    edges = slice.map(function (value, index) { return ({
                        cursor: offsetToCursor(startOffset + index),
                        node: value,
                    }); });
                    firstEdge = edges[0];
                    lastEdge = edges[edges.length - 1];
                    lowerBound = after ? afterOffset + 1 : 0;
                    upperBound = before ? Math.min(beforeOffset, totalCount) : totalCount;
                    return [2 /*return*/, {
                            edges: edges,
                            pageInfo: {
                                startCursor: firstEdge ? firstEdge.cursor : null,
                                endCursor: lastEdge ? lastEdge.cursor : null,
                                hasPreviousPage: last !== null ? startOffset > lowerBound : false,
                                hasNextPage: first !== null ? endOffset < upperBound : false,
                                count: count,
                            },
                        }];
            }
        });
    });
}
exports.paginate = paginate;
//# sourceMappingURL=paginate.js.map